@@ -8,6 +8,7 @@ from collections.abc import Mapping
 from contextlib import contextmanager
 from copy import copy, deepcopy
 from functools import wraps
+from json import JSONEncoder
 from json import dumps as json_dumps
 from json import loads as json_loads
 from os import PathLike
@@ -28,8 +29,27 @@ JSON = ("json",)
 PYTHON = ("py",)
 
 
-class Dumper(SafeDumper):  # pylint: disable=C0115
-    def increase_indent(self, flow: Optional[bool] = False, indentless: Optional[bool] = False):
+class JsonEncoder(JSONEncoder):
+    """
+    JSON encoder for Config.
+
+    """
+
+    def default(self, o: Any) -> Any:
+        if isinstance(o, Variable):
+            return o.value
+        if hasattr(o, "__json__"):
+            return o.__json__()
+        return super().default(o)
+
+
+class YamlDumper(SafeDumper):
+    """
+    YAML Dumper for Config.
+
+    """
+
+    def increase_indent(self, flow: bool = False, indentless: bool = False):  # pylint: disable=W0235
         return super().increase_indent(flow, indentless)
 
 
@@ -79,13 +99,20 @@ class ConfigParser(ArgumentParser):  # pylint: disable=C0115
                     self.add_argument(arg)
         if config is None:
             config = Config()
-        path = getattr(config, default_config, None) if default_config is not None else None
-        if path is not None:
-            warn(f"Config has 'default_config={path}' specified, its values will override values in Config")
-        parsed, _ = self.parse_known_args(args)
-        if path is not None:
-            config = config.update(path)  # type: ignore
-        config = config.update(vars(parsed))  # type: ignore
+        parsed = vars(self.parse_args(args))
+
+        # parse the config file
+        if default_config is not None:
+            if default_config in parsed:
+                path = parsed[default_config]
+                warn(f"Config has 'default_config={path}' specified, its values will override values in Config")
+                # create a temp config to avoid issues when users inherit from Config
+                config = config.update(Config.load(path))  # type: ignore
+            else:
+                raise ValueError(f"default_config is set to {default_config}, but not found in args")
+
+        # parse the command line arguments
+        config = config.update(parsed)  # type: ignore
         return config  # type: ignore
 
     parse_config = parse
@@ -101,33 +128,111 @@ class ConfigParser(ArgumentParser):  # pylint: disable=C0115
 class Variable:
     r"""
     Mutable wrapper for immutable objects.
+
+    Example:
+    ```python
+    >>> v = Variable(1)
+    >>> n = v
+    >>> v, n
+    (1, 1)
+    >>> v += 1
+    >>> v, n
+    (2, 2)
+    >>> v.value = 3
+    >>> v, n
+    (3, 3)
+    >>> v.set(4)
+    >>> v, n
+    (4, 4)
+    >>> v = 5
+    >>> v, n
+    (5, 4)
+    >>> f'4 < {v}'
+    '4 < 5'
+    >>> v = Variable('hello')
+    >>> f'{v}, world!'
+    'hello, world!'
+    >>> v += ', world!'
+    >>> v
+    'hello, world!'
+
+    ```
     """
 
+    wrap_type: bool = True
     storage: List[Any]
 
     def __init__(self, value):
         self.storage = [value]
 
+    @property  # type: ignore
+    def __class__(self):
+        return self.value.__class__ if self.wrap_type else type(self)
+
+    @property
+    def value(self):
+        """
+        actual object stored in the Variable
+        """
+
+        return self.storage[0]
+
+    @value.setter
+    def value(self, value):
+        """
+        assign value to object stored in the Variable
+        """
+
+        self.storage[0] = self._get_value(value)
+
+    @property
+    def dtype(self):
+        """
+        data type of Variable
+        """
+
+        return self.value.__class__
+
+    def get(self):
+        """
+        alias of value
+        """
+
+        return self.value
+
+    def set(self, value):
+        """
+        alias of value.setter
+        """
+
+        self.value = value
+
+    @staticmethod
+    def _get_value(obj):
+        if isinstance(obj, Variable):
+            return obj.value
+        return obj
+
     def __getattr__(self, attr):
         return getattr(self.value, attr)
 
     def __lt__(self, other) -> bool:
-        return self.value < self._get(other)
+        return self.value < self._get_value(other)
 
     def __le__(self, other) -> bool:
-        return self.value <= self._get(other)
+        return self.value <= self._get_value(other)
 
     def __eq__(self, other) -> bool:
-        return self.value == self._get(other)
+        return self.value == self._get_value(other)
 
     def __ne__(self, other) -> bool:
-        return self.value != self._get(other)
+        return self.value != self._get_value(other)
 
     def __ge__(self, other) -> bool:
-        return self.value >= self._get(other)
+        return self.value >= self._get_value(other)
 
     def __gt__(self, other) -> bool:
-        return self.value > self._get(other)
+        return self.value > self._get_value(other)
 
     def __index__(self) -> int:
         return int(self.value)
@@ -139,58 +244,94 @@ class Variable:
         return abs(self.value)
 
     def __add__(self, other):
-        return Variable(self.value + self._get(other))
+        return Variable(self.value + self._get_value(other))
 
     def __radd__(self, other):
-        return Variable(self._get(other) + self.value)
+        return Variable(self._get_value(other) + self.value)
+
+    def __iadd__(self, other):
+        self.value += self._get_value(other)
+        return self
 
     def __and__(self, other):
-        return Variable(self.value & self._get(other))
+        return Variable(self.value & self._get_value(other))
 
     def __rand__(self, other):
-        return Variable(self._get(other) & self.value)
+        return Variable(self._get_value(other) & self.value)
+
+    def __iand__(self, other):
+        self.value &= self._get_value(other)
+        return self
 
     def __floordiv__(self, other):
-        return Variable(self.value // self._get(other))
+        return Variable(self.value // self._get_value(other))
 
     def __rfloordiv__(self, other):
-        return Variable(self._get(other) // self.value)
+        return Variable(self._get_value(other) // self.value)
+
+    def __ifloordiv__(self, other):
+        self.value //= self._get_value(other)
+        return self
 
     def __mod__(self, other):
-        return Variable(self.value % self._get(other))
+        return Variable(self.value % self._get_value(other))
 
     def __rmod__(self, other):
-        return Variable(self._get(other) % self.value)
+        return Variable(self._get_value(other) % self.value)
+
+    def __imod__(self, other):
+        self.value %= self._get_value(other)
+        return self
 
     def __mul__(self, other):
-        return Variable(self.value * self._get(other))
+        return Variable(self.value * self._get_value(other))
 
     def __rmul__(self, other):
-        return Variable(self._get(other) * self.value)
+        return Variable(self._get_value(other) * self.value)
+
+    def __imul__(self, other):
+        self.value *= self._get_value(other)
+        return self
 
     def __matmul__(self, other):
-        return Variable(self.value @ self._get(other))
+        return Variable(self.value @ self._get_value(other))
 
     def __rmatmul__(self, other):
-        return Variable(self._get(other) @ self.value)
+        return Variable(self._get_value(other) @ self.value)
+
+    def __imatmul__(self, other):
+        self.value @= self._get_value(other)
+        return self
 
     def __pow__(self, other):
-        return Variable(self.value ** self._get(other))
+        return Variable(self.value ** self._get_value(other))
 
     def __rpow__(self, other):
-        return Variable(self._get(other) ** self.value)
+        return Variable(self._get_value(other) ** self.value)
+
+    def __ipow__(self, other):
+        self.value **= self._get_value(other)
+        return self
 
     def __truediv__(self, other):
-        return Variable(self.value / self._get(other))
+        return Variable(self.value / self._get_value(other))
 
     def __rtruediv__(self, other):
-        return Variable(self._get(other) / self.value)
+        return Variable(self._get_value(other) / self.value)
+
+    def __itruediv__(self, other):
+        self.value /= self._get_value(other)
+        return self
 
     def __sub__(self, other):
-        return Variable(self.value - self._get(other))
+        return Variable(self.value - self._get_value(other))
 
     def __rsub__(self, other):
-        return Variable(self._get(other) - self.value)
+        return Variable(self._get_value(other) - self.value)
+
+    def __isub__(self, other):
+        self.value -= self._get_value(other)
+        return self
 
     def __copy__(self):
         return Variable(self.value)
@@ -198,6 +339,9 @@ class Variable:
     def __deepcopy__(self, memo: Optional[Mapping] = None):
         return Variable(copy(self.value))
 
+    def __format__(self, format_spec):
+        return self.value if isinstance(self, str) else format(self.value, format_spec)
+
     def __repr__(self):
         return repr(self.value)
 
@@ -221,7 +365,7 @@ class Variable:
         ```
         """
 
-        self.storage[0] = cls(self.storage[0])
+        self.value = cls(self.value)
         return self
 
     def int(self) -> int:
@@ -269,23 +413,64 @@ class Variable:
 
         return self.to(str)
 
-    @property
-    def value(self):
+    @contextmanager
+    def unwraped(self):
         """
-        actual object stored in the Variable
-        """
-        return self.storage[0]
+        Context manager which temporarily unwrap the Variable.
 
-    @staticmethod
-    def _get(obj):
-        if isinstance(obj, Variable):
-            return obj.value
-        return obj
+        Example:
+        ```python
+        >>> v = Variable(1)
+        >>> isinstance(v, int)
+        True
+        >>> with v.unwraped():
+        ...    isinstance(v, int)
+        False
+
+        ```
+        """
+
+        wrap_type = self.wrap_type
+        self.wrap_type = False
+        try:
+            yield self
+        finally:
+            self.wrap_type = wrap_type
 
 
 class OrderedDict(OrderedDict_):
     """
-    Default OrderedDict with attributes
+    Default OrderedDict with attribute-style access.
+
+    Works best with `Variable` objects.
+
+    Example:
+    ```python
+    >>> d = OrderedDict()
+    >>> d.d = 1013
+    >>> d['d']
+    1013
+    >>> d['i'] = 1031
+    >>> d.i
+    1031
+    >>> d.a = Variable(1)
+    >>> d.b = d.a
+    >>> d.a, d.b
+    (1, 1)
+    >>> d.a += 1
+    >>> d.a, d.b
+    (2, 2)
+    >>> d.a = 3
+    >>> d.a, d.b
+    (3, 3)
+    >>> d.a = Variable('hello')
+    >>> f"{d.a}, world!"
+    'hello, world!'
+    >>> d.a = d.a + ', world!'
+    >>> d.b
+    'hello, world!'
+
+    ```
     """
 
     default_factory: Optional[Callable]
@@ -301,13 +486,12 @@ class OrderedDict(OrderedDict_):
                     f"default_factory={default_factory} should be of type Callable, but got {type(default_factory)}"
                 )
         self.setattr("indent", 2)
-        self.init(*args, **kwargs)
+        self._init(*args, **kwargs)
 
-    def init(self, *args, **kwargs) -> None:
+    def _init(self, *args, **kwargs) -> None:
         r"""
-        Initialise values for OrderedDict.
+        Initialise values from arguments for OrderedDict.
         This method is called in `__init__`.
-        You can overwrite this method to avoid call `super().__init__()` in `__init__`.
 
         Args:
             *args: [(key1, value1), (key2, value2)].
@@ -417,9 +601,12 @@ class OrderedDict(OrderedDict_):
         if isinstance(value, str):
             try:
                 value = literal_eval(value)
-            except (ValueError, SyntaxError):
+            except (TypeError, ValueError, SyntaxError):
                 pass
-        super().__setitem__(name, value)
+        if name in self and isinstance(self[name], Variable):
+            self[name].set(value)
+        else:
+            super().__setitem__(name, value)
 
     __setitem__ = set
     __setattr__ = set
@@ -556,7 +743,7 @@ class OrderedDict(OrderedDict_):
         ```
         """
 
-        return cls(**self)
+        return cls(**{k: v.value if isinstance(v, Variable) else v for k, v in self.items()})
 
     convert = to
     dict = to
@@ -631,16 +818,19 @@ class OrderedDict(OrderedDict_):
             other = other.items()
         if not isinstance(other, Iterable):
             raise TypeError(f"other={other} should be of type Mapping, Iterable or PathStr, but got {type(other)}")
-        return self.empty(
+
+        return self.empty_like(
             **{key: value for key, value in other if key not in self or self[key] != value}  # type: ignore
         )
 
     diff = difference
 
-    def intersection(self, other: Union[Mapping, Iterable, PathStr]) -> Mapping:
+    def intersection(self, other: Union[Mapping, Iterable, PathStr]) -> OrderedDict:
         r"""
         Intersection between OrderedDict values and other.
 
+        `inter` is an alias of this method.
+
         Args:
             other (Mapping | Iterable | PathStr): Other values to join.
 
@@ -666,7 +856,11 @@ class OrderedDict(OrderedDict_):
             other = other.items()
         if not isinstance(other, Iterable):
             raise TypeError(f"other={other} should be of type Mapping, Iterable or PathStr, but got {type(other)}")
-        return self.empty(**{key: value for key, value in other if key in self and self[key] == value})  # type: ignore
+        return self.empty_like(
+            **{key: value for key, value in other if key in self and self[key] == value}  # type: ignore
+        )
+
+    inter = intersection
 
     def copy(self) -> OrderedDict:
         r"""
@@ -745,7 +939,7 @@ class OrderedDict(OrderedDict_):
 
         empty = cls()
         empty.clear()
-        empty.init(*args, **kwargs)
+        empty._init(*args, **kwargs)
         return empty
 
     def empty_like(self, *args, **kwargs):
@@ -819,6 +1013,8 @@ class OrderedDict(OrderedDict_):
         ```
         """
 
+        if "cls" not in kwargs:
+            kwargs["cls"] = JsonEncoder
         if "indent" not in kwargs:
             kwargs["indent"] = self.getattr("indent")
         return json_dumps(self.to(dict), *args, **kwargs)
@@ -888,7 +1084,7 @@ class OrderedDict(OrderedDict_):
         """
 
         if "Dumper" not in kwargs:
-            kwargs["Dumper"] = Dumper
+            kwargs["Dumper"] = YamlDumper
         if "indent" not in kwargs:
             kwargs["indent"] = self.getattr("indent")
         return yaml_dump(self.to(dict), *args, **kwargs)  # type: ignore
@@ -931,7 +1127,7 @@ class OrderedDict(OrderedDict_):
             return self.yaml(file=file, *args, **kwargs)  # type: ignore
         if extension in JSON:
             return self.json(file=file, *args, **kwargs)  # type: ignore
-        raise FileError(f"file {file} should be in {JSON} or {YAML}, but got {extension}")
+        raise FileError(f"file {file} should be in {JSON} or {YAML}, but got {extension}")  # type: ignore
 
     @classmethod
     def load(cls, file: File, method: Optional[str] = None, *args, **kwargs) -> OrderedDict:  # pylint: disable=W1113
@@ -970,7 +1166,7 @@ class OrderedDict(OrderedDict_):
             try:
                 yield file
             finally:
-                file.close()
+                file.close()  # type: ignore
         elif isinstance(file, (IO,)):
             yield file
         else:
@@ -1051,7 +1247,7 @@ class NestedDict(OrderedDict):
         self.setattr("default_mapping", NestedDict)
         super().__init__(*args, default_factory=default_factory, **kwargs)
 
-    def init(self, *args, **kwargs) -> None:
+    def _init(self, *args, **kwargs) -> None:
         for key, value in args:
             self.set(key, value, convert_mapping=True)
         for key, value in kwargs.items():
@@ -1158,6 +1354,33 @@ class NestedDict(OrderedDict):
     __setitem__ = set
     __setattr__ = set
 
+    def __contains__(self, name: str) -> bool:  # type: ignore
+        r"""
+        Determine if NestedDict contains name.
+
+        Args:
+            name (str): Key name.
+
+        Example:
+        ```python
+        >>> d = NestedDict(**{'a': 1, 'b.c': 2})
+        >>> 'a' in d
+        True
+        >>> 'b.c' in d
+        True
+        >>> 'b' in d
+        True
+        >>> 'd' in d
+        False
+
+        ```
+        """
+
+        while self.getattr("delimiter") in name:
+            name, rest = name.split(self.getattr("delimiter"), 1)
+            self, name = self[name], rest  # pylint: disable=W0642
+        return super().__contains__(name)
+
     def pop(self, name: str, default: Optional[Any] = None) -> Any:
         r"""
         Pop value from NestedDict.
@@ -1303,14 +1526,121 @@ class NestedDict(OrderedDict):
 
         ret = cls()
         for k, v in self.items():
+            if isinstance(v, Variable):
+                v = v.value
             if isinstance(v, OrderedDict):
-                v = v.convert(cls)
+                v = v.to(cls)
             ret[k] = v
         return ret
 
     convert = to
     dict = to
 
+    def difference(  # pylint: disable=W0221
+        self, other: Union[Mapping, Iterable, PathStr], recursive: bool = True
+    ) -> NestedDict:
+        r"""
+        Difference between NestedDict values and other.
+
+        `diff` is an alias of this method.
+
+        Args:
+            other (Mapping | Iterable | PathStr): Other values to compare.
+            recursive (bool): Whether to compare recursively. Defaults to True.
+
+        Example:
+        ```python
+        >>> d = NestedDict(**{'a': 1, 'b.c': 2, 'b.d': 3})
+        >>> n = {'a': 1, 'b.c': 3, 'b.d': 3, 'e': 4}
+        >>> d.difference(n).to(dict)
+        {'b': {'c': 3}, 'e': 4}
+        >>> d.difference(n, recursive=False).to(dict)
+        {'b': {'c': 3, 'd': 3}, 'e': 4}
+        >>> l = [('a', 1), ('d', 4)]
+        >>> d.difference(l).to(dict)
+        {'d': 4}
+        >>> d.difference(1)
+        Traceback (most recent call last):
+        TypeError: other=1 should be of type Mapping, Iterable or PathStr, but got <class 'int'>
+
+        ```
+        """
+
+        if isinstance(other, (PathLike, str, bytes)):
+            other = self.load(other)
+        if isinstance(other, (Mapping,)):
+            other = self.empty_like(**other).items()
+        if not isinstance(other, Iterable):
+            raise TypeError(f"other={other} should be of type Mapping, Iterable or PathStr, but got {type(other)}")
+
+        @wraps(self.difference)
+        def difference(this: NestedDict, that: Iterable) -> Mapping:
+            ret = {}
+            for key, value in that:
+                if key not in this:
+                    ret[key] = value
+                elif isinstance(this[key], NestedDict) and recursive:
+                    ret[key] = this[key].difference(value)
+                elif this[key] != value:
+                    ret[key] = value
+            return ret
+
+        return self.empty_like(**difference(self, other))  # type: ignore
+
+    diff = difference
+
+    def intersection(  # pylint: disable=W0221
+        self, other: Union[Mapping, Iterable, PathStr], recursive: bool = True
+    ) -> NestedDict:
+        r"""
+        Intersection between NestedDict values and other.
+
+        `inter` is an alias of this method.
+
+        Args:
+            other (Mapping | Iterable | PathStr): Other values to join.
+            recursive (bool): Whether to compare recursively. Defaults to True.
+
+        Example:
+        ```python
+        >>> d = NestedDict(**{'a': 1, 'b.c': 2, 'b.d': 3})
+        >>> n = {'a': 1, 'b.c': 3, 'b.d': 3, 'e': 4}
+        >>> d.intersection(n).to(dict)
+        {'a': 1, 'b': {'d': 3}}
+        >>> d.intersection(n, recursive=False).to(dict)
+        {'a': 1}
+        >>> l = [('a', 1), ('d', 4)]
+        >>> d.intersection(l).to(dict)
+        {'a': 1}
+        >>> d.intersection(1)
+        Traceback (most recent call last):
+        TypeError: other=1 should be of type Mapping, Iterable or PathStr, but got <class 'int'>
+
+        ```
+        """
+
+        if isinstance(other, (PathLike, str, bytes)):
+            other = self.load(other)
+        if isinstance(other, (Mapping,)):
+            other = self.empty_like(**other).items()
+        if not isinstance(other, Iterable):
+            raise TypeError(f"other={other} should be of type Mapping, Iterable or PathStr, but got {type(other)}")
+
+        @wraps(self.intersection)
+        def intersection(this: NestedDict, that: Iterable) -> Mapping:
+            ret = {}
+            for key, value in that:
+                if key in this:
+                    if isinstance(this[key], NestedDict) and recursive:
+                        ret[key] = this[key].intersection(value)
+                    elif this[key] == value:
+                        ret[key] = value
+            return ret
+
+        return self.empty_like(**intersection(self, other))  # type: ignore
+
+    inter = intersection
+
 
 class DefaultDict(NestedDict):
     """
@@ -1467,12 +1797,14 @@ class Config(NestedDict):
 
         return super().pop(name, default)
 
-    def freeze(self, recursive: Optional[bool] = True) -> Config:
+    def freeze(self, recursive: bool = True) -> Config:
         r"""
         Freeze the config.
 
+        `lock` is an alias of this method.
+
         Args:
-            recursive (Optional[bool]): freeze all sub-configs recursively. Defaults to True.
+            recursive (bool): freeze all sub-configs recursively. Defaults to True.
 
         Example:
         ```python
@@ -1497,12 +1829,16 @@ class Config(NestedDict):
             freeze(self)
         return self
 
-    def defrost(self, recursive: Optional[bool] = True) -> Config:
+    lock = freeze
+
+    def defrost(self, recursive: bool = True) -> Config:
         r"""
         Defrost the config.
 
+        `unlock` is an alias of this method.
+
         Args:
-            recursive (Optional[bool]): defrost all sub-configs recursively. Defaults to True.
+            recursive (bool): defrost all sub-configs recursively. Defaults to True.
 
         Example:
         ```python
@@ -1531,6 +1867,34 @@ class Config(NestedDict):
             defrost(self)
         return self
 
+    unlock = defrost
+
+    @contextmanager
+    def unlocked(self):
+        """
+        Context manager which temporarily unlocks the config.
+
+        Example:
+        ```python
+        >>> c = Config()
+        >>> c.freeze().to(dict)
+        {}
+        >>> with c.unlocked():
+        ...     c['i.d'] = 1013
+        >>> c.to(dict)
+        {'i': {'d': 1013}}
+
+        ```
+        """
+
+        was_frozen = self.getattr("frozen")
+        try:
+            self.defrost()
+            yield self
+        finally:
+            if was_frozen:
+                self.freeze()
+
     def parse(
         self,
         args: Optional[Iterable[str]] = None,
